/*
 * ---------------------------------------------------------------------------
 *   GKrellM who plugin for GKrellM
 *   Copyright (C) 2001 Ernesto Novillo
 *   ernie@cs.ualberta.ca
 *   www.cs.ualberta.ca/~ernie
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * ---------------------------------------------------------------------------
 */

/* Aug 1, 2001: Fixed segfault caused by GK.normal_font reference in
|  get_users().  Converted to GKrellM 1.2.0 functions.  Bill Wilson.
*/

#include <stdio.h>
#include <string.h>
#include <gkrellm/gkrellm.h>


#define	CONFIG_NAME	"GKrellM who"
#define	STYLE_NAME	"gkrellmwho"



/* configuration tab */
static GtkWidget *spn_freq;
static GtkWidget *txt_no_users;
static GtkWidget *spn_scroll;

static void create_who_tab( GtkWidget* );
static void apply_who_config( void );
static void load_who_config( gchar* );
static void save_who_config( FILE* );


static Monitor *monitor;

/* GKrellM panel */
static Panel *panel;
static Decal *decal_text1;
static gint style_id;



/* gkrellmwho specific */
#define VERSION		"0.4"
#ifdef L_cuserid
#define MAX_USER_LEN	L_cuserid+1
#else
#define MAX_USER_LEN	10
#endif

#define MAX_USERS	10	/* set this statically to avoid realloc() */
#define USERS_LEN	MAX_USERS*MAX_USER_LEN + 1
#define COMMAND		"w -hs | sort | grep -v $LOGNAME"
#define DEF_NONE_MSG	"no users"
static gchar users[USERS_LEN];	// list of users
static int users_xoff;		// length of users string in pixels
void get_users( void );


/* option variables */
int	SCROLL_SPEED	= 2;
int	FREQUENCY	= 1;
char*	NONE_MSG	= NULL;


/* debug info */
#ifdef DEBUG
#include <time.h>
#define DEBUG_FILE	"/usr/brule2/grad/ernie/gkwho.dbg"
FILE* gkwhodbg=NULL;
time_t t;
#endif



void get_users( void )  {
    /*
       Build a string containing the list of currently logged on users, except
       the current users, as indicated by env variable LOGNAME.
    */
    static FILE* p;
    static int user_count, prev_user_idx, last_idx;
    static char user_tmp[MAX_USER_LEN];

#ifdef DEBUG
    if( (gkwhodbg=fopen( DEBUG_FILE, "a" )) )  {
	time( &t );
	fprintf( gkwhodbg, "%sPREVIOUS VALUES:\n", ctime( &t ) );
	fprintf( gkwhodbg, "users      == %s\n", users );
	fprintf( gkwhodbg, "user_count == %d\n", user_count );
	fprintf( gkwhodbg, "\n" );
	fclose( gkwhodbg );
    } else {
	fprintf( stderr, "%s:%d: Could not open debug file.\n", __FILE__,
		__LINE__ );
    }
#endif

    /* open pipe and execute `w -hs | sort | grep -v $LOGNAME` */
    if( !(p=popen( COMMAND, "r" )) )  {
	strcpy( users, "COULD NOT OPEN PIPE" );
	return;
    }

    user_count = 0;
    prev_user_idx = 0;
    last_idx = 0;
    users[0] = '\0';
    user_tmp[0] = '\0';

    /* read in user names from the pipe */
    while( !feof( p ) && user_count<MAX_USERS )  {
	/* get user name */
	fscanf( p, "%s %*[^\n]", user_tmp );
	/* add it only if it's not there already */
	if( strncmp( &users[prev_user_idx], user_tmp, strlen( user_tmp ) ) )  {
	    prev_user_idx = last_idx;
	    last_idx += sprintf( &users[last_idx], "%s ", user_tmp );
	    user_count++;
	}
    }
    pclose( p );

    /* validate empty users list */
    if( !user_count )  {
	strcpy( users, NONE_MSG );
    } else {
	users[last_idx] = '\0';
    }

    /* set pixel length of users string */
    users_xoff = -gdk_string_width( gkrellm_default_font(1), users );

#ifdef DEBUG
    if( (gkwhodbg=fopen( DEBUG_FILE, "a" )) )  {
	time( &t );
	fprintf( gkwhodbg, "%sNEW VALUES:\n", ctime( &t ) );
	fprintf( gkwhodbg, "users      == %s\n", users );
	fprintf( gkwhodbg, "user_count == %d\n", user_count );
	fprintf( gkwhodbg, "\n" );
	fclose( gkwhodbg );
    } else {
	fprintf( stderr, "%s:%d: Could not open debug file.\n", __FILE__,
		__LINE__ );
    }
#endif

}


static gint panel_expose_event( GtkWidget *widget, GdkEventExpose *ev )  {
    gdk_draw_pixmap( widget->window, widget->style->fg_gc[GTK_WIDGET_STATE
	    (widget)], panel->pixmap, ev->area.x, ev->area.y, ev->area.x,
	    ev->area.y, ev->area.width, ev->area.height );
    return FALSE;
}


static void update_plugin()  {
    static gint x_scroll, w, freq;

    if( GK.minute_tick )  {
	freq++;
	if( freq==FREQUENCY )  {
	    get_users();
	    freq = 0;
	}
    }

    if( w==0 )  {
	w = gkrellm_chart_width();
	x_scroll = w;
    }

    if( x_scroll > users_xoff )  {
	x_scroll -= SCROLL_SPEED;
    } else {
	x_scroll = w;
    }
    decal_text1->x_off = x_scroll;

    gkrellm_draw_decal_text( panel, decal_text1, users, w - x_scroll );
    gkrellm_draw_panel_layers( panel );
}



static void create_plugin( GtkWidget *vbox, gint first_create )
{
    Style *style;

    if( first_create )  {
	  panel = gkrellm_panel_new0();
	  get_users();
    }

    style = gkrellm_meter_style( style_id );

    /* Create a text decal that will be used to scroll text.  Make it
    |  the full panel width (minus the margins).  Position it at top border
    |  and left margin of the style.
    */
    decal_text1 = gkrellm_create_decal_text( panel, "Ay",
				gkrellm_meter_textstyle( style_id ), style,
	   			-1, -1, -1 );

    gkrellm_panel_configure( panel, NULL, style );
    gkrellm_panel_create( vbox, monitor, panel);

    if( first_create )  {
	gtk_signal_connect( GTK_OBJECT (panel->drawing_area), "expose_event",
			    (GtkSignalFunc)panel_expose_event, NULL );
    }
}


/* The monitor structure tells GKrellM how to call the plugin routines. */
static Monitor plugin_mon = {
    CONFIG_NAME,        /* Name, for config tab. */
    0,			/* Id,  0 if a plugin */
    create_plugin,	/* The create function */
    update_plugin,	/* The update function */
    create_who_tab,	/* The config tab create function */
    apply_who_config,	/* Apply the config function */
    save_who_config,	/* Save user config */
    load_who_config,	/* Load user config */
    STYLE_NAME,		/* config keyword */
    NULL,		/* Undefined 2 */
    NULL,		/* Undefined 1 */
    NULL,		/* private */
    MON_FS,		/* Insert plugin before this monitor */ 
    NULL,		/* Handle if a plugin, filled in by GKrellM */
    NULL		/* path if a plugin, filled in by GKrellM */
};


/* All GKrellM plugins must have one global routine named init_plugin()
|  which returns a pointer to a filled in monitor structure.
*/
Monitor * init_plugin()  {
    /* If this call is made, the background and krell images for this plugin
    |  can be custom themed by putting bg_meter.png or krell.png in the
    |  subdirectory STYLE_NAME of the theme directory.  Text colors (and
    |  other things) can also be specified for the plugin with gkrellmrc
    |  lines like:  StyleMeter  STYLE_NAME.textcolor orange black shadow
    |  If no custom themeing has been done, then all above calls using
    |  style_id will be equivalent to style_id = DEFAULT_STYLE_ID.
    */
    style_id = gkrellm_add_meter_style( &plugin_mon, STYLE_NAME );

    /* validate NONE_MSG option */
    if( !NONE_MSG )  {
	NONE_MSG = strdup( DEF_NONE_MSG );
    }
	monitor = &plugin_mon;
    return( &plugin_mon );
}



static void load_who_config( gchar* arg )  {
    gchar option[17], value[USERS_LEN];
    value[USERS_LEN-1] = '\0';

    if( sscanf( arg, "%s %[^\n]", option, value ) == 2 )  {
        if( !strcmp( option, "scroll_speed" ) )  {
	    sscanf( value, "%d\n", &SCROLL_SPEED );
	}
        if( !strcmp( option, "update_frequency" ) )  {
	    sscanf( value, "%d\n", &FREQUENCY );
	}
        if( !strcmp( option, "no_users_msg" ) )  {
	    free( NONE_MSG );
	    NONE_MSG = strdup( value );
	}
    }
}



static void apply_who_config( void )  {

    free( NONE_MSG );
    NONE_MSG  = strdup( gtk_entry_get_text( GTK_ENTRY(txt_no_users) ) );
    FREQUENCY = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(spn_freq) );
    SCROLL_SPEED =
	gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON(spn_scroll) );

}



static void save_who_config( FILE* f )  {
    fprintf( f, "%s scroll_speed %d\n", STYLE_NAME, SCROLL_SPEED );
    fprintf( f, "%s update_frequency %d\n", STYLE_NAME, FREQUENCY );
    fprintf( f, "%s no_users_msg %s\n", STYLE_NAME, NONE_MSG );
}



static void create_who_tab( GtkWidget *tab )
{
    GtkWidget *nbk_who;
    GtkWidget *tbl_opt;
    GtkWidget *lbl_scroll;
    GtkWidget *lbl_no_users;
    GtkWidget *lbl_freq;
    GtkWidget *box_scroll;
    GtkObject *spn_scroll_adj;
    GtkWidget *box_freq;
    GtkObject *spn_freq_adj;
    GtkWidget *tab_options;
    GtkWidget *scr_info;
    GtkWidget *txt_info;
    GtkWidget *tab_info;
    GtkWidget *lbl_about;
    GtkWidget *tab_about;

    nbk_who = gtk_notebook_new ();
    gtk_notebook_set_tab_pos( GTK_NOTEBOOK(nbk_who), GTK_POS_TOP );
    gtk_box_pack_start( GTK_BOX( tab ), nbk_who, TRUE, TRUE, 0 );

    gtk_widget_set_name (nbk_who, "nbk_who");
    gtk_widget_ref (nbk_who);
    gtk_widget_show (nbk_who);

    tbl_opt = gtk_table_new (3, 2, FALSE);
    gtk_widget_set_name (tbl_opt, "tbl_opt");
    gtk_widget_ref (tbl_opt);
    gtk_widget_show (tbl_opt);
    gtk_container_add (GTK_CONTAINER (nbk_who), tbl_opt);
    gtk_container_set_border_width (GTK_CONTAINER (tbl_opt), 10);
    gtk_table_set_row_spacings (GTK_TABLE (tbl_opt), 10);
    gtk_table_set_col_spacings (GTK_TABLE (tbl_opt), 10);

    txt_no_users = gtk_entry_new_with_max_length (100);
    gtk_widget_set_name (txt_no_users, "txt_no_users");
    gtk_widget_ref (txt_no_users);
    gtk_widget_show (txt_no_users);
    gtk_table_attach (GTK_TABLE (tbl_opt), txt_no_users, 1, 2, 0, 1,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
    gtk_entry_set_text (GTK_ENTRY (txt_no_users), NONE_MSG );

    lbl_scroll = gtk_label_new (_("Scroll speed:"));
    gtk_widget_set_name (lbl_scroll, "lbl_scroll");
    gtk_widget_ref (lbl_scroll);
    gtk_widget_show (lbl_scroll);
    gtk_table_attach (GTK_TABLE (tbl_opt), lbl_scroll, 0, 1, 1, 2,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
    gtk_label_set_justify (GTK_LABEL (lbl_scroll), GTK_JUSTIFY_RIGHT);
    gtk_misc_set_alignment (GTK_MISC (lbl_scroll), 1, 0.5);

    lbl_no_users = gtk_label_new (_("No users string:"));
    gtk_widget_set_name (lbl_no_users, "lbl_no_users");
    gtk_widget_ref (lbl_no_users);
    gtk_widget_show (lbl_no_users);
    gtk_table_attach (GTK_TABLE (tbl_opt), lbl_no_users, 0, 1, 0, 1,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
    gtk_label_set_justify (GTK_LABEL (lbl_no_users), GTK_JUSTIFY_RIGHT);
    gtk_misc_set_alignment (GTK_MISC (lbl_no_users), 1, 0.5);

    lbl_freq = gtk_label_new (_("Update frequency (min):"));
    gtk_widget_set_name (lbl_freq, "lbl_freq");
    gtk_widget_ref (lbl_freq);
    gtk_widget_show (lbl_freq);
    gtk_table_attach (GTK_TABLE (tbl_opt), lbl_freq, 0, 1, 2, 3,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
    gtk_label_set_justify (GTK_LABEL (lbl_freq), GTK_JUSTIFY_RIGHT);
    gtk_misc_set_alignment (GTK_MISC (lbl_freq), 1, 0.5);

    box_scroll = gtk_hbox_new (FALSE, 0);
    gtk_widget_set_name (box_scroll, "box_scroll");
    gtk_widget_ref (box_scroll);
    gtk_widget_show (box_scroll);
    gtk_table_attach (GTK_TABLE (tbl_opt), box_scroll, 1, 2, 1, 2,
		    (GtkAttachOptions) (GTK_FILL),
		    (GtkAttachOptions) (GTK_FILL), 0, 0);

    spn_scroll_adj = gtk_adjustment_new (SCROLL_SPEED, 1, 10, 1, 1, 1);
    spn_scroll = gtk_spin_button_new (GTK_ADJUSTMENT (spn_scroll_adj), 1, 0);
    gtk_widget_set_name (spn_scroll, "spn_scroll");
    gtk_widget_ref (spn_scroll);
    gtk_widget_show (spn_scroll);
    gtk_box_pack_start (GTK_BOX (box_scroll), spn_scroll, FALSE, TRUE, 0);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (spn_scroll),
	    GTK_UPDATE_IF_VALID);
    gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (spn_scroll), TRUE);
    gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spn_scroll), TRUE);

    box_freq = gtk_hbox_new (FALSE, 0);
    gtk_widget_set_name (box_freq, "box_freq");
    gtk_widget_ref (box_freq);
    gtk_widget_show (box_freq);
    gtk_table_attach (GTK_TABLE (tbl_opt), box_freq, 1, 2, 2, 3,
		    (GtkAttachOptions) (GTK_FILL),
		    (GtkAttachOptions) (GTK_FILL), 0, 0);

    spn_freq_adj = gtk_adjustment_new (FREQUENCY, 1, 15, 1, 1, 1);
    spn_freq = gtk_spin_button_new (GTK_ADJUSTMENT (spn_freq_adj), 1, 0);
    gtk_widget_set_name (spn_freq, "spn_freq");
    gtk_widget_ref (spn_freq);
    gtk_widget_show (spn_freq);
    gtk_box_pack_start (GTK_BOX (box_freq), spn_freq, FALSE, TRUE, 0);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (spn_freq),
	    GTK_UPDATE_IF_VALID);
    gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (spn_freq), TRUE);
    gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spn_freq), TRUE);

    tab_options = gtk_label_new (_("Options"));
    gtk_widget_set_name (tab_options, "tab_options");
    gtk_widget_ref (tab_options);
    gtk_widget_show (tab_options);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (nbk_who),
	    gtk_notebook_get_nth_page (GTK_NOTEBOOK (nbk_who), 0),
	    tab_options);

    scr_info = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_set_name (scr_info, "scr_info");
    gtk_widget_ref (scr_info);
    gtk_widget_show (scr_info);
    gtk_container_add (GTK_CONTAINER (nbk_who), scr_info);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scr_info),
	    GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

    txt_info = gtk_text_new (NULL, NULL);
    gtk_widget_set_name (txt_info, "txt_info");
    gtk_widget_ref (txt_info);
    gtk_widget_show (txt_info);
    gtk_container_add (GTK_CONTAINER (scr_info), txt_info);
    GTK_WIDGET_UNSET_FLAGS (txt_info, GTK_CAN_FOCUS);
    gtk_text_insert (GTK_TEXT (txt_info), NULL, NULL, NULL,
       "GKrellM who scrolls a list of currently logged on "
       "users, excluding the invoking user.\n\n"
       "It gets the current list of users by issuing the shell command:\n\n\t"
       COMMAND"\n\n"
       "Options:\n"
       "\t* No users string: String to scroll when no other users are logged "
       "on.\n"
       "\t* Scroll speed: Speed at which the list of users scrolls through the"
       " panel.\n"
       "\t* Update frequency (min): Frequency at which the shell command is "
       "invoked.",
		   419);

    tab_info = gtk_label_new (_("Info"));
    gtk_widget_set_name (tab_info, "tab_info");
    gtk_widget_ref (tab_info);
    gtk_widget_show (tab_info);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (nbk_who),
	    gtk_notebook_get_nth_page (GTK_NOTEBOOK (nbk_who), 1), tab_info);

    lbl_about = gtk_label_new (
	    "GKrellM who Plugin v"VERSION"\n\n"
	    "Copyright (C) 2001  Ernesto Novillo\n"
	    "ernie@cs.ualberta.ca\n"
	    "http://www.cs.ualberta.ca/~ernie\n\n"
	    "Released under the GNU Lesser General Public License" );
    gtk_widget_set_name (lbl_about, "lbl_about");
    gtk_widget_ref (lbl_about);
    gtk_widget_show (lbl_about);
    gtk_container_add (GTK_CONTAINER (nbk_who), lbl_about);
    gtk_misc_set_alignment (GTK_MISC (lbl_about), 0.5, 0.45);

    tab_about = gtk_label_new (_("About"));
    gtk_widget_set_name (tab_about, "tab_about");
    gtk_widget_ref (tab_about);
    gtk_widget_show (tab_about);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (nbk_who),
	    gtk_notebook_get_nth_page (GTK_NOTEBOOK (nbk_who), 2), tab_about);

}
